#Based on https://gist.github.com/weipah/19bfdb14aab253e3f109
#September 2016
function Upload-Log {
    param (
        [parameter(Mandatory=$True,Position=1)] [ValidateScript({ Test-Path -PathType Leaf $_ })] [String] $FilePath,
        [parameter(Mandatory=$True,Position=2)] [System.URI] $RESTURL,
        [parameter(Mandatory=$True,Position=3)] [System.URI] $IP,
        [parameter(Mandatory=$True,Position=4)] [System.URI] $filename,
        [parameter(Mandatory=$True,Position=5)] [System.URI] $md5,
        [parameter(Mandatory=$True,Position=6)] [System.URI] $filesize,
        [parameter(Mandatory=$False,Position=7)] [System.URI] $TDID

    )

	# CONST
	$CODEPAGE = "UTF-8" # alternatives are ASCII, UTF-8

	# Read file byte-by-byte
    $fileBin = [System.IO.File]::ReadAllBytes($FilePath)

    # Convert byte-array to string
	$enc = [System.Text.Encoding]::GetEncoding($CODEPAGE)

	$fileEnc = $enc.GetString($fileBin)

	# We need a boundary (something random() will do best)
    $boundary = [System.Guid]::NewGuid().ToString()

	# Linefeed character
    $LF = "`r`n"

	# Build up URI for the API-call
	$uri = $RESTURL

    # write-host "uri=$uri"
    # write-host "TDID=$TDID"
    # write-host "IP=$IP"


	# Build Body for our form-data manually since PS does not support multipart/form-data out of the box
    if(-Not [string]::IsNullOrWhiteSpace($TDID)) {
        # send form data with TDID
        $bodyLines = (
            "--$boundary",
            "Content-Disposition: form-data; name=`"upfile`"; filename=`"$filename`"",
            "Content-Type: application/octet-stream$LF",
            $fileEnc,
            "--$boundary",
            "Content-Disposition: form-data; name=`"md5_client`"$LF",
            $md5,
            "--$boundary",
            "Content-Disposition: form-data; name=`"fw_ip`"$LF",
            $IP,
            "--$boundary",
            "Content-Disposition: form-data; name=`"fw_tdid`"$LF",
            $TDID,
            "--$boundary",
            "Content-Disposition: form-data; name=`"upfile_size`"$LF",
            $filesize,
            "--$boundary--$LF"
         ) -join $LF

    }
    else {
        # send original form data with IP
        $bodyLines = (
            "--$boundary",
            "Content-Disposition: form-data; name=`"upfile`"; filename=`"$filename`"",
            "Content-Type: application/octet-stream$LF",
            $fileEnc,
            "--$boundary",
            "Content-Disposition: form-data; name=`"md5_client`"$LF",
            $md5,
            "--$boundary",
            "Content-Disposition: form-data; name=`"fw_ip`"$LF",
            $IP,
            "--$boundary",
            "Content-Disposition: form-data; name=`"upfile_size`"$LF",
            $filesize,
            "--$boundary--$LF"
         ) -join $LF
    }

    # write-host "headers=$bodyLines"

    try {
        # Submit form-data with Invoke-RestMethod-Cmdlet
        $response=Invoke-RestMethod -Uri $uri -Method Post -ContentType "multipart/form-data; boundary=`"$boundary`"" -Body $bodyLines
        return $response
    }
    # In case of emergency...
    catch [System.Net.WebException] {
        $err = ( "ERROR: REST-API-Call failed for '$URL': $_" )
        write-host $err
        return $err
    }
    #write-host $bodyLines
}


#based on https://gist.github.com/hsmalley/3836383
function Get-IniContent ($filePath)
{
    $ini = @{}
    switch -regex -file $FilePath
    {
        "^\[(.+)\]" # Section
        {
            $section = $matches[1]
            $ini[$section] = @{}
            $CommentCount = 0
        }
        "^(;.*)$" # Comment
        {
            $value = $matches[1]
            $CommentCount = $CommentCount + 1
            $name = Comment + $CommentCount
            $ini[$section][$name] = $value
        }
        "^(?!;)(.+?)s*=s*(.*)" # Key
        {
            $name,$value = $matches[1..2]
            $ini[$section][$name] = $value
        }
    }
    return $ini
}

#based on http://stackoverflow.com/questions/32360801/split-text-file-into-smaller-files-based-on-size-windows
function Split-FileByLine
{
    [CmdletBinding()]
    Param
    (
        [Parameter(Mandatory = $true, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
        [string[]]$FileName,

        [Parameter(ValueFromPipelineByPropertyName = $true)]
        [string]$OutPath = (Get-Location -PSProvider FileSystem).Path,

        [Parameter(Mandatory = $true, ValueFromPipelineByPropertyName = $true)]
        [long]$MaxFileSize,

        [Parameter(ValueFromPipelineByPropertyName = $true)]
        [string]$Encoding = 'Default'
    )

    Begin
    {
        # Scriptblocks for common tasks
        $DisposeInFile = {
            Write-Verbose 'Disposing StreamReader'
            $InFile.Close()
            $InFile.Dispose()
        }

        $DisposeOutFile = {
            Write-Verbose 'Disposing StreamWriter'
            $OutFile.Flush()
            $OutFile.Close()
            $OutFile.Dispose()
        }

        $NewStreamWriter = {
            Write-Verbose 'Creating StreamWriter'
            $OutFileName = Join-Path -Path $OutPath -ChildPath (
                '{0}_part_{1}{2}' -f [System.IO.Path]::GetFileNameWithoutExtension($_), $Counter, [System.IO.Path]::GetExtension($_)
            )

            $OutFile = New-Object -TypeName System.IO.StreamWriter -ArgumentList (
                $OutFileName,
                $false,
                $FileEncoding
            ) -ErrorAction Stop
            $OutFile.AutoFlush = $true
            Write-Verbose "Writing new file: $OutFileName"
        }
    }

    Process
    {
        if($Encoding -eq 'Default')
        {
            # Set default encoding
            $FileEncoding = [System.Text.Encoding]::Default
        }
        else
        {
            # Try to set user-specified encoding
            try
            {
                $FileEncoding = [System.Text.Encoding]::GetEncoding($Encoding)
            }
            catch
            {
                throw "Not valid encoding: $Encoding"
            }
        }

        Write-Verbose "Input file: $FileName"
        Write-Verbose "Output folder: $OutPath"

        if(!(Test-Path -Path $OutPath -PathType Container)){
            Write-Verbose "Folder doesn't exist, creating: $OutPath"
            $null = New-Item -Path $OutPath -ItemType Directory -ErrorAction Stop
        }

        $FileName | ForEach-Object {
            # Open input file
            $InFile = New-Object -TypeName System.IO.StreamReader -ArgumentList (
                $_,
                $FileEncoding
            ) -ErrorAction Stop
            Write-Verbose "Current file: $_"

            $Counter = 0
            $OutFile = $null

            # Read lines from input file
            while(($line = $InFile.ReadLine()) -ne $null)
            {
                if($OutFile -eq $null)
                {
                    # No output file, create StreamWriter
                    . $NewStreamWriter
                }
                else
                {
                    if($OutFile.BaseStream.Length -ge $MaxFileSize)
                    {
                        # Output file reached size limit, closing
                        Write-Verbose "OutFile lenght: $($InFile.BaseStream.Length)"
                        . $DisposeOutFile
                        $Counter++
                        . $NewStreamWriter
                    }
                }

                # Write line to the output file
                $OutFile.WriteLine($line)
            }

            Write-Verbose "Finished processing file: $_"
            # Close open files and cleanup objects
            . $DisposeOutFile
            . $DisposeInFile
        }
    }
}